//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "analyse.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#define OBLIGATOIRE true
#define FACULTATIF  false
// Note : NBMSGERR est dfini dans analyse.h
// Erreurs lexicales & syntaxiques. ERR_AUCUNE dfini dans analyse.h
#define ERR_IDENTIFICATEUR_INCORRECT                  1
#define ERR_DEBUT_LISTE_ATTENDU                       5
#define ERR_SEPARATEUR_OU_FIN_ARGUMENTS_ATTENDUS      6
#define ERR_SEP_OU_FIN_CONTRAINTES_REGLE_ATTENDUS     7
#define ERR_FIN_INATTENDUE                            8
#define ERR_IDENTIFICATEUR_REGLE_NON_AUTORISE         9
#define ERR_IDENTIFICATEUR_REGLE_INCONNU             10
#define ERR_SEP_OU_FIN_CONTRAINTES_QUESTION_ATTENDUS 11
#define ERR_SUITE_SIGNATURE_REGLE_INCORRECTE         12
#define ERR_IDENTIFICATEUR_REGLE_PREDEF_INTERDIT     13
// Note : NBIDENTIFICATEURSREGLEPREDEFINIE est dfini dans analyse.h
//---------------------------------------------------------------------------
// pour asIdentificateurReglePredefinie
#define IRP_EXCLUSION 1
//---------------------------------------------------------------------------
// Pour la grammaire voir base.cpp
//---------------------------------------------------------------------------
const AnsiString asErreur[NBMSGERR]={ "Analyse %s[%s] : Aucune erreur.",
                                      "Identificateur attendu.",
                                      "Identificateur de rgle attendu.",
                                      "Identificateur de variable attendu.",
                                      "Identificateur d'lment attendu.",
                                      "'(' attendue.",
                                      "',' ou ')' attendue.",
                                      "',' ou ';' ou '.' attendu.",
                                      "Fin inattendue.",
                                      "Identificateur de rgle non autoris.",
                                      "Identificateur de rgle '%s' inconnu.",
                                      "',' ou '?' attendu.",
                                      "':' ou ';' ou '.' attendu.",
                                      "Identificateur de rgle prdfinie '#' non autorise."};
const AnsiString asCibleAnalyse[NBCIBLESANALYSE]={"Base","Question"};
const AnsiString asTypeMsgAnalyse[NBTYPESMSGANALYSE]={"Lex&Syn","Sem"};

//---------------------------------------------------------------------------
bool analyse::PasseCaractereSuivant(int &Pos)
 {
  if (Pos<asSource.Length())
   {
    Pos++;
    return true;
   }
  else
   {
    NumErreur=ERR_FIN_INATTENDUE;
    PosErreur=Pos;
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::PasseSeparateurs(int &Pos)
 {
  bool Commentaire;
  while((Pos<asSource.Length())&&
        ((Commentaire=(asSource[Pos]==DebutCommentaire))|| // v0.2
         (asSource[Pos]=='\r')||
         (asSource[Pos]=='\n')||
         (asSource[Pos]=='\t')||
         (asSource[Pos]==' ')))
   {
    if (Commentaire)  // v0.2
     while ((Pos<asSource.Length())&&
            (asSource[Pos]!='\r')&&
            (asSource[Pos]!='\n'))
      Pos++;
    else
     Pos++;
   }
  return true; // Permet son usage en fonction
 }
//---------------------------------------------------------------------------
bool analyse::EstIdentificateurReglePredefinie(int &Pos, AnsiString &asIdentificateur)
 {
  int Taille=0;
  for( int i=1;
       (!Taille)&&
       (i<=NBIDENTIFICATEURSREGLEPREDEFINIE); // Attention : asIdentificateurReglePredefinie commence  1
       i++)
   if (asSource.SubString(Pos, TAILLEMAXIDENTIFICATEURREGLEPREDEFINIE).AnsiPos(asIdentificateurReglePredefinie[i])==1)
    {
     asIdentificateur=asIdentificateurReglePredefinie[i];
     Pos+=(Taille=asIdentificateurReglePredefinie[i].Length());
    }
  return Taille>0;
 }
//---------------------------------------------------------------------------
bool analyse::EstIdentificateur(int &Pos, type_identificateur &Type, AnsiString &asIdentificateur)
 {
  int i, Taille=asSource.Length();
  bool EstMajuscule=false,
       EstMinuscule=false,
       InitialeEnMajuscule=false,
       InitialeEnMinuscule=false,
       ToutEnMajuscules=true,
       ToutEnMinuscules=true;
  PasseSeparateurs(Pos);
  if (EstIdentificateurReglePredefinie(Pos, asIdentificateur))
   {
    Type=tiRegle;
    PasseSeparateurs(Pos);
    return true;
   }
  for( i=Pos;
       (i<=Taille)&&
       ((EstMajuscule=(asSource[i]>='A')&&
                      (asSource[i]<='Z'))||
        (EstMinuscule=(asSource[i]>='a')&&
                      (asSource[i]<='z'))||
        ((asSource[i]>='0')&&
         (asSource[i]<='9'))||
        (asSource[i]==Souligne));
       i++)
   {
    if (!(i-Pos))
     {
      InitialeEnMajuscule=EstMajuscule;
      InitialeEnMinuscule=(!EstMajuscule)&&EstMinuscule;
     }
    ToutEnMajuscules&=EstMajuscule||(!EstMinuscule); // non-lettres ignores
    ToutEnMinuscules&=EstMinuscule||(!EstMajuscule); // non-lettres ignores
   }
  if (i>Taille)
   {
    NumErreur=ERR_FIN_INATTENDUE;
    PosErreur=i-Pos;
    return false;
   }
  if ((i-Pos)&& // Dans ce qui suit on affecte la nature si indfinie
      ((((!Type)||(Type==tiRegle))&&InitialeEnMinuscule&&ToutEnMinuscules&&(Type=tiRegle))||
       (((!Type)||(Type==tiVariable))&&ToutEnMajuscules&&(Type=tiVariable))||
       (((!Type)||(Type==tiElement))&&InitialeEnMajuscule&&(!ToutEnMajuscules)&&(Type=tiElement))))
   {
    asIdentificateur=asSource.SubString(Pos, i-Pos);
    Pos+=i-Pos;
    PasseSeparateurs(Pos);
    return true;
   }
  else
   {
    NumErreur=ERR_IDENTIFICATEUR_INCORRECT+Type;
    PosErreur=Pos-1;
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::EstArgument(int &Pos, argument *Argument) // Argument->NatureIdentificateur doit tre  niIndefinie
 {
  PasseSeparateurs(Pos);
  int PosAvantIdentificateur=Pos-1;
  if (EstIdentificateur(Pos, Argument->Type, Argument->asIdentificateur))
   {
    if (Argument->Type==tiRegle)
     {
      NumErreur=ERR_IDENTIFICATEUR_REGLE_NON_AUTORISE;
      PosErreur=PosAvantIdentificateur;
      return false;
     }
    Argument->PosTxtSrc=PosAvantIdentificateur;
    Argument->LgrTxtSrc=Pos-PosAvantIdentificateur-1;
    PasseSeparateurs(Pos);
    return true;
   }
  else
   return false;
 }
//---------------------------------------------------------------------------
bool analyse::EstPartieArguments(int &Pos, argument **Arguments) // Pr-requis : NE PAS CREER *Arguments  l'appel, *Argument doit tre  NULL !
 {
  argument **NvArgument=Arguments; // Pointeur sur le nouvel argument de la liste Arguments
  bool FinArguments=false, Ok=true;
  PasseSeparateurs(Pos);
  if (asSource[Pos]!=DebutListeArguments)
   {
    NumErreur=ERR_DEBUT_LISTE_ATTENDU;
    PosErreur=Pos-1;
    return false;
   }
  else
   if (!PasseCaractereSuivant(Pos)) return false;
  PasseSeparateurs(Pos);
  if (asSource[Pos]!=FinListeArguments)
   {
    while(Ok&&(!FinArguments)&&(Ok=EstArgument(Pos,(*NvArgument)=new argument)))
     if (Ok=((asSource[Pos]==SeparateurArguments)||
             (FinArguments=(asSource[Pos]==FinListeArguments))))
      if (!PasseCaractereSuivant(Pos))
       {
        if (*Arguments) { delete *Arguments; *Arguments=NULL; }
        return false;
       }
      else
       NvArgument=&((*NvArgument)->Svt);
   }
  else
   if (!PasseCaractereSuivant(Pos))
    {
     if (*Arguments) { delete *Arguments; *Arguments=NULL; }
     return false;
    }
  if (Ok)
   {
    PasseSeparateurs(Pos);
    return true;
   }
  else
   {
    if (*Arguments) { delete *Arguments; *Arguments=NULL; }
    if (!NumErreur) // Si dfini alors identificateur incorrect sinon sparateur ou fin de liste
     {
      NumErreur=ERR_SEPARATEUR_OU_FIN_ARGUMENTS_ATTENDUS;
      PosErreur=Pos-1;
     }
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::EstContrainte(int &Pos, contrainte *Contrainte) // Pr-requis : Contrainte doit tre cr avant
 {
  argument *Arguments=NULL; // Il s'agit de TOUS les arguments
  PasseSeparateurs(Pos);
  Contrainte->PosTxtSrc=Pos-1; // Permet de reprer l'identificateur de rgle pour la liaison aprs analyse
  if (EstIdentificateur(Pos, tiRegle, Contrainte->asIdentificateurRegle)&&
      EstPartieArguments(Pos, &Arguments)) // "(" [ Arguments ] ")"
   {
    Contrainte->Arguments=Arguments;
    Contrainte->LgrTxtSrc=Pos-Contrainte->PosTxtSrc-1;
    return true;
   }
  else
   return false;
 }
//---------------------------------------------------------------------------
bool analyse::EstPartieContraintes(int &Pos, bool &DerniereRegle, definition *Definition, bool Question)
 {
  bool Ok=true, ContrainteRequise=true;
  contrainte *NvContrainte;
  DerniereRegle=false;
  PasseSeparateurs(Pos);
  if (Question||(asSource[Pos]==SeparateurSignatureContraintes)) // Si contraintes prsentes (Analyse d'une base)
   {
    if ((!Question)&& // On passe le sparateur de signature dans le cas de l'analyse d'une base
        (!PasseCaractereSuivant(Pos))) return false;
    while(ContrainteRequise&&(Ok=EstContrainte(Pos, NvContrainte=new contrainte)))
     if ((asSource[Pos]!=SeparateurContraintes)&&
         (Question||(asSource[Pos]!=SeparateurRegles)&&
                    (asSource[Pos]!=FinListeRegles))&&
         ((!Question)||(asSource[Pos]!=FinQuestion)))
      {
       delete NvContrainte; NvContrainte=NULL;
       NumErreur=Question? ERR_SEP_OU_FIN_CONTRAINTES_QUESTION_ATTENDUS:
                           ERR_SEP_OU_FIN_CONTRAINTES_REGLE_ATTENDUS;
       PosErreur=Pos-1;
       return false;
      }
     else
      {
       Definition->AjouteContrainte(NvContrainte);
       if (ContrainteRequise=(asSource[Pos]==SeparateurContraintes))
        {
         if (!PasseCaractereSuivant(Pos)) return false;
         PasseSeparateurs(Pos);
        }
      }
   }
  if (Ok)
   {
    if (!(DerniereRegle=((Question&&
                          (asSource[Pos]==FinQuestion))||
                         (asSource[Pos]==FinListeRegles))))
     {
      if ((!Question)&&(asSource[Pos]!=SeparateurRegles))
       {
        NumErreur=ERR_SUITE_SIGNATURE_REGLE_INCORRECTE;
        PosErreur=Pos-1;
        return false;
       }
      if (!PasseCaractereSuivant(Pos)) return false;
      PasseSeparateurs(Pos);
      return true;
     }
    else
     return true;
   }
  else
   {
    delete NvContrainte; NvContrainte=NULL;
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::EstDefinitionRegle(int &Pos, bool &DerniereRegle, regle *Regle)
 {
  definition *NvDefinition=NULL;
  argument *Arguments=NULL; // Il s'agit de TOUS les arguments de la dfinition
  int PosIdentificateurRegle=Pos-1;
  if (EstIdentificateurReglePredefinie(Pos, Regle->asIdentificateur)) // v2.9 : Identificateur de Rgle Prdfinie INTERDIT
   {
    Pos=PosIdentificateurRegle+1; // On remet la position avant l'identificateur
    PosErreur=PosIdentificateurRegle;
    asPrmErreur=Regle->asIdentificateur;
    NumErreur=ERR_IDENTIFICATEUR_REGLE_PREDEF_INTERDIT;
    return false;
   }
  if (EstIdentificateur(Pos, tiRegle, Regle->asIdentificateur)&&
      EstPartieArguments(Pos, &Arguments)&& // "(" [ Arguments ] ")"
      EstPartieContraintes(Pos, DerniereRegle, NvDefinition=new definition, CA_BASE)) // [":" constraintes] ";"
   {
    NvDefinition->PosTxtSrc=PosIdentificateurRegle;
    NvDefinition->LgrTxtSrc=Pos-PosIdentificateurRegle-1;
    Regle->AjouteDefinition(NvDefinition, Arguments);
    return true;
   }
  else
   {
    if (NvDefinition) { delete NvDefinition; NvDefinition=NULL; }
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::EstListeRegles()
 {
  regle *NvRegle;
  int Pos=1;
  bool DerniereRegle;
  while(EstDefinitionRegle(Pos, DerniereRegle, NvRegle=new regle)&&(!DerniereRegle))
   Base->AjouteRegle(NvRegle);  // En fait la rgle peut exister dj alors dans ce cas on ajoute la dfinition sinon la rgle
  if (DerniereRegle) // Dernire rgle
   {
    Base->AjouteRegle(NvRegle); // En fait la rgle peut exister dj alors dans ce cas on ajoute la signature sinon la rgle
    return true;
   }
  else // Sinon... Il y a une erreur
   {
    delete NvRegle; NvRegle=NULL; 
    return false;
   }
 }
//---------------------------------------------------------------------------
bool analyse::IdentifieReglesBase()
 {
  bool Ok=true;
  regle *RegleCrt;
  signature *SignatureCrt;
  definition *DefinitionCrt;
  contrainte *ContrainteCrt;
  argument *ArgumentCrt;
  for(RegleCrt=Base->Regles; Ok&&RegleCrt; RegleCrt=RegleCrt->Svt)
   for(SignatureCrt=RegleCrt->Signatures; Ok&&SignatureCrt; SignatureCrt=SignatureCrt->Svt)
    for(DefinitionCrt=SignatureCrt->Definitions; Ok&&DefinitionCrt; DefinitionCrt=DefinitionCrt->Svt)
     for(ContrainteCrt=DefinitionCrt->Contraintes; Ok&&ContrainteCrt; ContrainteCrt=ContrainteCrt->Svt)
      if (!(ContrainteCrt->Regle=ReglesPredefinies.Regle(ContrainteCrt->asIdentificateurRegle))) // v0.3 (rgle prdfinie)
       {
        ContrainteCrt->Regle=Base->Regles->Regle(ContrainteCrt->asIdentificateurRegle);
        if (!ContrainteCrt->Regle)
         {
          PosErreur=ContrainteCrt->PosTxtSrc;
          asPrmErreur=ContrainteCrt->asIdentificateurRegle;
          NumErreur=ERR_IDENTIFICATEUR_REGLE_INCONNU;
          return false;
         }
       }
   return true;
 }
//---------------------------------------------------------------------------
bool analyse::IdentifieReglesRecherche()
 {
  bool Ok=true;
  contrainte *ContrainteCrt;
  argument *ArgumentCrt;
  for(ContrainteCrt=Recherche->Question->Contraintes; Ok&&ContrainteCrt; ContrainteCrt=ContrainteCrt->Svt)
   if (!(ContrainteCrt->Regle=ReglesPredefinies.Regle(ContrainteCrt->asIdentificateurRegle))) // v0.3 (rgle prdfinie)
    {
     ContrainteCrt->Regle=Base->Regles->Regle(ContrainteCrt->asIdentificateurRegle);
     if (!ContrainteCrt->Regle)
      {
       PosErreur=ContrainteCrt->PosTxtSrc;
       asPrmErreur=ContrainteCrt->asIdentificateurRegle;
       NumErreur=ERR_IDENTIFICATEUR_REGLE_INCONNU;
       return false;
      }
    }
  return true;
 }
//---------------------------------------------------------------------------
bool analyse::EstQuestion()
 {
  return EstPartieContraintes(1, false, Recherche->Question, CA_QUESTION);
 }
//---------------------------------------------------------------------------

