//---------------------------------------------------------------------------
#include "vclx.h"
#pragma hdrstop
//---------------------------------------------------------------------------
#include <stdio.h>
//---------------------------------------------------------------------------
#ifdef PRFD_WINDOWS
#include <SysUtils.hpp>
#include <ShellApi.hpp>
#include <Registry.hpp>
#endif
//---------------------------------------------------------------------------
#ifdef PRFD_LINUX
#include <stdio.h>
#include <SysUtils.hpp>
#endif
//---------------------------------------------------------------------------
#include "main_f.h"
#include "propos_f.h"
#include "analyse.h"
#include "arbre.h"
#include "winlinux.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#ifdef PRFD_WINDOWS
#pragma resource "*.dfm"
#endif
#ifdef PRFD_LINUX
#pragma resource "*.xfm"
#endif
//---------------------------------------------------------------------------
// Pour les rubrique d'affichage/masquage des structures arborescentes
#define MASQUER   0
#define RETABLIR  1
#define BASE      0
#define RECHERCHE 1
// Pour TTreeNode::Expand(...) et TTreeNode::Collapse(...)
#define ALL_THE_NODES true
//---------------------------------------------------------------------------
TFormMain *FormMain=NULL;
//---------------------------------------------------------------------------
const AnsiString asIndicateursModification[4]={"","*","~","*~"},
                 asCleExtProFond="\\.profond",
                 asCleProFond="\\ProFond",
                 asProFond="ProFond";

//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
  : TForm(Owner)
{
#ifdef PRFD_WINDOWS
 MemoBase->HideSelection=false;
 MemoReponse->HideSelection=false;
#endif
}
//---------------------------------------------------------------------------
void TFormMain::AfficheIndicateursModification()
 {
  if (asNomFichier=="")
   Caption=asProFond+asIndicateursModification[Modification+2*ModificationDepuisAnalyse];
  else
   Caption=asProFond+" - "+asNomFichier+asIndicateursModification[Modification+2*ModificationDepuisAnalyse];
 }
//---------------------------------------------------------------------------
void TFormMain::DefinitModificationProjet(bool Modification) // Change l'tat de modification et rafrachit l'indicateur (*)
 {
  this->Modification=Modification;
  AfficheIndicateursModification();
 }
//---------------------------------------------------------------------------
void TFormMain::DefinitModificationProjetDepuisAnalyse(bool ModificationDepuisAnalyse) // Change l'tat de modification et rafrachit l'indicateur (~)
 {
  this->ModificationDepuisAnalyse=ModificationDepuisAnalyse;
  AfficheIndicateursModification();
 }
//---------------------------------------------------------------------------
void TFormMain::AfficheErreur(const AnsiString asTypeAnalyse, int PosErreur, int NumErreur, const AnsiString asMsgErreur, bool Question)
 {

  int NbCaracteresPasses, NumLigne;
  if (Question)
   {
    StatusBar->SimpleText= Format( "Analyse %s[%s] Caractre %d Erreur %d : %s",
                                   ARRAYOFCONST(( asTypeAnalyse,
                                                  asCibleAnalyse[Question],
                                                  PosErreur+1,
                                                  NumErreur,
                                                  asMsgErreur)));
    ComboBoxQuestion->SelStart=PosErreur;
    ComboBoxQuestion->SelLength=0;
    ComboBoxQuestion->SetFocus();
   }
  else
   {
    for( NbCaracteresPasses=0, NumLigne=0;
         NbCaracteresPasses<PosErreur;
         NumLigne++)
      NbCaracteresPasses+=MemoBase->Lines->Strings[NumLigne].Length()+2;
    StatusBar->SimpleText= Format( "Analyse %s[%s] Ligne %d Caractre %d Erreur %d : %s",
                                   ARRAYOFCONST(( asTypeAnalyse,
                                                  asCibleAnalyse[Question],
                                                  NumLigne,
                                                  1+PosErreur-NbCaracteresPasses+
                                                  MemoBase->Lines->Strings[NumLigne-1].Length()+2,
                                                  NumErreur,
                                                  asMsgErreur)));
    MemoBase->SelStart=PosErreur;
    MemoBase->SelLength=0;
    MemoBase->SetFocus();
   }
 }
//---------------------------------------------------------------------------
void TFormMain::AnalyseBase()
 {
  if (MemoBase->Lines->Text.IsEmpty()) return; // A DEFINIR : prvoir une base vide quand mme
  TreeViewBase->Items->Clear();
  if (Base) { delete Base; Base=NULL; }
  if (Base=new base)
   {
    analyse Analyse(MemoBase->Lines->Text);
    if (!Analyse.EstListeRegles())
     {
      AfficheErreur( asTypeMsgAnalyse[TMA_LEXSYN],
                     Analyse.PosErreur,
                     Analyse.NumErreur,
                     asErreur[Analyse.NumErreur],
                     CA_BASE);
     }
    else
     {
      StatusBar->SimpleText=Format( asErreur[ERR_AUCUNE],
                                    ARRAYOFCONST(( asTypeMsgAnalyse[TMA_LEXSYN],
                                                   asCibleAnalyse[CA_BASE])));
      if (Analyse.IdentifieReglesBase())
       {
        Base->DefinitParents();
        Base->ReleveObjets();
        CreeArbreBase();
        ProjetModifieDepuisAnalyse=false;
       }
      else
       {
        AfficheErreur( asTypeMsgAnalyse[TMA_SEM],
                       Analyse.PosErreur,
                       Analyse.NumErreur,
                       Format( asErreur[Analyse.NumErreur],
                               ARRAYOFCONST((Analyse.asPrmErreur))),
                       CA_BASE);
       }
     }
   }
 }
//---------------------------------------------------------------------------
void TFormMain::RechercheSolutions()
 {
  int SolutionTrouvee=Recherche->Cherche( MemoReponse->Lines,
                                          ActionDetail->Checked,
                                          ActionInterrompre->Checked);
  if (ActionInterrompre->Checked&&SolutionTrouvee)
   {
    ActionPoursuivre->Enabled=true;
    ActionArreter->Enabled=true;
   }
  else
   {
    ActionPoursuivre->Enabled=false;
    ActionArreter->Enabled=false;
    delete Recherche; Recherche=NULL;
   }
 }
//---------------------------------------------------------------------------
void TFormMain::AnalyseQuestion()
 {
  if (ComboBoxQuestion->Text.IsEmpty()) return;
  if (!Base)
   {
    AfficheMessage("\
La base de connaissances est vide.\r\n\
Utilisez d'abord la commande \"Projet | Analyser\" avant de poser une question.",
                            "Impossible d'valuer la question",
                            tmAvertissement, 0, 0);
    return;
   }
  else
   {
    if (Recherche) { delete Recherche; Recherche=NULL; }
    Recherche=new recherche(PRFD_TEXT(ComboBoxQuestion));
    analyse Analyse(Recherche, PRFD_TEXT(ComboBoxQuestion));
    if (!Analyse.EstQuestion())
     {
      AfficheErreur( asTypeMsgAnalyse[TMA_LEXSYN],
                     Analyse.PosErreur,
                     Analyse.NumErreur,
                     asErreur[Analyse.NumErreur],
                     CA_QUESTION);
     }
    else
     {
      StatusBar->SimpleText=Format( asErreur[ERR_AUCUNE],
                                    ARRAYOFCONST(( asTypeMsgAnalyse[TMA_LEXSYN],
                                                   asCibleAnalyse[CA_QUESTION])));
      if (Analyse.IdentifieReglesRecherche())
       {
        Recherche->DefinitParents();
        Recherche->ReleveObjets();
        CreeArbreQuestion(Recherche);
        Base->InitialiseIndicateursRecherche(asVide);
         {
          AnsiString asQuestionPosee=PRFD_TEXT(ComboBoxQuestion);
          int Pos=ComboBoxQuestion->Items->IndexOf(asQuestionPosee);
          if (Pos)
           {
            if (Pos>-1) ComboBoxQuestion->Items->Delete(Pos);
            ComboBoxQuestion->Items->Insert(0, asQuestionPosee);
            ComboBoxQuestion->ItemIndex=0;
           }
#ifdef PRFD_WINDOWS
          EnregistreHistoriqueQuestions(); // v0.3.2
#endif
         }
        MemoReponse->Lines->Add(PRFD_TEXT(ComboBoxQuestion));
        RechercheSolutions();
       }
      else
       {
        AfficheErreur( asTypeMsgAnalyse[TMA_SEM],
                       Analyse.PosErreur,
                       Analyse.NumErreur,
                       Format( asErreur[Analyse.NumErreur],
                               ARRAYOFCONST((Analyse.asPrmErreur))),
                       CA_QUESTION);
       }
     }
   }
 }
//---------------------------------------------------------------------------
void TFormMain::CreeArbreBase()
 {
  (new arbre(TreeViewBase))->CreeArbre(StatusBar);
 }
//---------------------------------------------------------------------------
void TFormMain::CreeArbreQuestion(recherche *Recherche)
 {
  (new arbre(Recherche, TreeViewRecherche, ActionNumeroter->Checked))->CreeArbre(StatusBar); // ATTENTION : l'instance est rfrence par Recherche->Arbre lors de sa cration
 }
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormShow(TObject *Sender)
{
 if (ParamCount()>0)
  if (FileExists(ParamStr(1)))
   Ouvrir(ParamStr(1), moEcraser);
  else
   AfficheMessage("Le fichier \""+ParamStr(1)+"\" n'existe pas.", asProFond, tmErreur, 0, 0);
}
//---------------------------------------------------------------------------

bool TFormMain::ConfirmeAvecDemandeEnregistrementModifications()
 {
  if (ProjetModifie)
   switch(AfficheMessage( "Le projet "+
                          asNomFichier+
                          " a t modifi.\nVoulez-vous enregistrer les modifications ?",
                          "Avertissement",
                          tmQuestionFacultative, 0, 0))
    {
     case trOui: ActionEnregistrer->Execute();
     case trNon: return true;
     default:    return false;
    }
  else
   return true;
}
//---------------------------------------------------------------------------
void TFormMain::Ouvrir(const AnsiString asNomFichier, mode_ouverture Mode)
 {
  bool FinFichier=false;
#ifdef PRFD_WINDOWS
    LitHistoriqueQuestions(asNomFichier);
#endif
  if (Mode==moEcraser)
   {
    ActionNouveau->Execute();
    OuvertureEnCours=true;
    MemoBase->Lines->LoadFromFile(asNomFichier);
    OuvertureEnCours=false;
    this->asNomFichier=asNomFichier;
    AfficheIndicateursModification();
    return;
   }
  FILE *f=NULL;
  try
   {
    f=fopen(asNomFichier.c_str(), "rt");
    char CaractereLu='\x0';
    AnsiString asLigne;
    int NumLigne=0,
        NbCaracteresPasses,
        PosPtInsertion=(Mode==moAjouterEnTete)?0:MemoBase->SelStart;
    if (PosPtInsertion)
     for( NbCaracteresPasses=0;
          NbCaracteresPasses<PosPtInsertion;
          NumLigne++)
      NbCaracteresPasses+=MemoBase->Lines->Strings[NumLigne].Length()+2;
    do
     {
      fread(&CaractereLu, sizeof(CaractereLu), 1, f);
      if ((!(FinFichier=feof(f)))&&(CaractereLu!='\n'))
       asLigne=asLigne+CaractereLu;
      else
       {
        if (Mode!=moAjouterEnFin)
         MemoBase->Lines->Insert(NumLigne++, asLigne);
        else
         MemoBase->Lines->Append(asLigne);
        asLigne="";
       }
     }
    while(!FinFichier);
   }
  __finally
   {
    if (f) fclose(f);
   }
 }
//---------------------------------------------------------------------------
void __fastcall TFormMain::MemoBaseChange(TObject *Sender)
{
 if (OuvertureEnCours) return;
 if (!ProjetModifie) ProjetModifie=true;
 if (!ProjetModifieDepuisAnalyse) ProjetModifieDepuisAnalyse=true;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
 CanClose=ConfirmeAvecDemandeEnregistrementModifications();
}
//---------------------------------------------------------------------------
void TFormMain::Montre(int Position, int Longueur)
 {
  MemoBase->SelStart=Position;
  MemoBase->SelLength=Longueur;
 }
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
 // Initialisation des proprits de la fiche
 OuvertureEnCours=false;
#ifdef PRFD_WINDOWS
 IniFile=new TIniFile(ExtractFilePath(ParamStr(0))+"\\ProFond.ini"); // v0.3.2
 // Contrle de la base de registre sur le lien avec les fichiers de programme ProFond
 if (TRegistry *Reg=new TRegistry)
  {
   try
    {
     Reg->RootKey=HKEY_CLASSES_ROOT;
     if (!Reg->KeyExists(asCleExtProFond))
      {
       Reg->OpenKey(asCleExtProFond, true);
       Reg->WriteString(asVide, asProFond);
       Reg->CloseKey();
      }
     Reg->OpenKey(asCleProFond, true);
     Reg->WriteString(asVide, "Fichier source ProFond");
     Reg->OpenKey("DefaultIcon", true);
     Reg->WriteString(asVide, ParamStr(0)+",1");
     Reg->OpenKey("\\ProFond\\shell", true);
     Reg->OpenKey("open", true);
     Reg->OpenKey("command", true);
     Reg->WriteString(asVide, ParamStr(0)+AnsiString(" \"%1\""));
     Reg->CloseKey();
     Reg->CloseKey();
     Reg->CloseKey();
     Reg->CloseKey();
     Reg->CloseKey();
    }
   catch(...)
    {
     Application->MessageBox("Impossible d'ouvrir et/ou de mettre  jour la base de registres", asProFond.c_str(), MB_ICONHAND);
    }
   delete Reg;
  }
#endif
}
//---------------------------------------------------------------------------
// Evnements Action (v0.2)
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionNouveauExecute(TObject *Sender)
{
 if (!ConfirmeAvecDemandeEnregistrementModifications()) return;
 MemoBase->Lines->Clear();
 MemoReponse->Lines->Clear();
 ComboBoxQuestion->Text="";
 TreeViewBase->Items->Clear();
 TreeViewRecherche->Items->Clear();
 if (Recherche) { delete Recherche; Recherche=NULL; }
 if (Base) { delete Base; Base=NULL; }
 StatusBar->SimpleText="";
 asNomFichier="";
 ProjetModifie=false; ProjetModifieDepuisAnalyse=false;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionDetailExecute(TObject *Sender)
{
 ActionDetail->Checked^=true;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionOuvrirExecute(TObject *Sender)
{
 if (OpenDialog->Execute())
  Ouvrir(PRFD_FILENAME(OpenDialog), (mode_ouverture)((TComponent*)Sender)->Tag);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionEnregistrerExecute(TObject *Sender)
{
 if (asNomFichier=="")
  ActionEnregistrerSous->Execute();
 else
  {
   MemoBase->Lines->SaveToFile(asNomFichier);
   ProjetModifie=false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionAnalyserExecute(TObject *Sender)
{
 AnalyseBase();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionRepondreExecute(TObject *Sender)
{
 AnalyseQuestion();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionPoursuivreExecute(TObject *Sender)
{
 RechercheSolutions();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionArreterExecute(TObject *Sender)
{
 ActionPoursuivre->Enabled=false;
 ActionArreter->Enabled=false;
 MemoReponse->Lines->Add("Recherche arrte.");
 if (Recherche) { delete Recherche; Recherche=NULL; };
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionResultatEffacerExecute(TObject *Sender)
{
 MemoReponse->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionEnregistrerSousExecute(TObject *Sender)
{
 if (SaveDialog->Execute())
  {
   asNomFichier=SaveDialog->FileName;
   MemoBase->Lines->SaveToFile(asNomFichier);
#ifdef PRFD_WINDOWS
   EnregistreHistoriqueQuestions(); // v0.3.2
#endif
   ProjetModifie=false;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionInterrompreExecute(TObject *Sender)
{
 ActionInterrompre->Checked^=true;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionQuitterExecute(TObject *Sender)
{
 if (ConfirmeAvecDemandeEnregistrementModifications())
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionAProposExecute(TObject *Sender)
{
 AProposDeProFond();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionAideExecute(TObject *Sender)
{
#ifdef PRFD_WINDOWS
 ShellExecute( Handle,
               NULL,
               AnsiString(ExtractFilePath(ParamStr(0))+"\\html\\index.html").c_str(),
               NULL,
               AnsiString(ExtractFilePath(ParamStr(0))+"\\html").c_str(),
               SW_SHOWNORMAL);
#endif
#ifdef PRFD_LINUX
AfficheMessage("Pour plus d'informations sur l'utilisation et la rfrence de Profond, consulter le fichier /html/index.html", "Aide", tmInformation, 0, 0);
#endif
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionMontrerExecute(TObject *Sender)
{
 if (ProjetModifieDepuisAnalyse)
  {
   if (AfficheMessage( "\
Le projet a t modifi depuis la dernire analyse.\n\
Pour voir la correspondance dans le source, vous devez effectuer une nouvelle analyse.\n\
Remarque : un indicateur '~' s'affiche dans le titre de la fentre lorsqu'une analyse est requise.\n\n\
Souhaitez-vous le faire maintenant ?",
                                "Analyse requise", tmQuestionNecessaire, 0, 1)==trOui)
    ActionAnalyser->Execute();
   return;
  }
 TTreeNode *TreeNode=TreeViewBase->Selected;
 if (!TreeNode) return;
 switch(TreeNode->ImageIndex)
  {
   case IDB_CONTRAINTE:   if (PRFD_TEXT(TreeNode).AnsiPos(asLibelleNoeud[ND_CONTRAINTES]))
                           Montre( ((contrainte*)TreeNode->Data)->PosTxtSrc,
                                   ((contrainte*)TreeNode->Data)->LgrTxtSrcTotale());
                          else
                           Montre( ((contrainte*)TreeNode->Data)->PosTxtSrc,
                                   ((contrainte*)TreeNode->Data)->LgrTxtSrc);
                          break;
   case IDB_ELEMENTCNTR:
   case IDB_VARIABLECNTR:
   case IDB_ELEMENTDEFN:
   case IDB_VARIABLEDEFN:
   case IDB_ARGUMENTCNTR:
   case IDB_ARGUMENTDEFN: if (PRFD_TEXT(TreeNode).AnsiPos(asLibelleNoeud[ND_ARGUMENTS]))
                           Montre( ((argument*)TreeNode->Data)->PosTxtSrc,
                                   ((argument*)TreeNode->Data)->LgrTxtSrcTotale());
                          else
                           Montre( ((argument*)TreeNode->Data)->PosTxtSrc,
                                   ((argument*)TreeNode->Data)->LgrTxtSrc);
                          break;
   case IDB_QUESTION:
   case IDB_DEFINITION:   Montre( ((definition*)TreeNode->Data)->PosTxtSrc,
                                  ((definition*)TreeNode->Data)->LgrTxtSrc);
                          break;
   case IDB_REGLECNTR:    Montre( ((contrainte*)TreeNode->Data)->PosTxtSrc, // v0.3
                                  ((contrainte*)TreeNode->Data)->Regle->asIdentificateur.Length());
                          break;
   default: break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionBaseRetourAutoExecute(TObject *Sender)
{
 ActionBaseRetourAuto->Checked^=true;
 MemoBase->ScrollBars=ActionBaseRetourAuto->Checked?ssVertical:ssBoth;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionBaseEffacerExecute(TObject *Sender)
{
 if ( AfficheMessage( "tes-vous sr(e) de bien vouloir effacer TOUT le texte de la base ?",
                      "Confirmation d'effacement",
                      tmQuestionNecessaire, 1, 1)==trOui)
  {
   ProjetModifie=true; ProjetModifieDepuisAnalyse=true;
   MemoBase->Lines->Clear();
   TreeViewBase->Items->Clear();
   TreeViewRecherche->Items->Clear();
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionBasePoliceExecute(TObject *Sender)
{
 FontDialog->Font=MemoBase->Font;
 if (FontDialog->Execute())
  MemoBase->Font=FontDialog->Font;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionQuestionEffacerExecute(TObject *Sender)
{
 ComboBoxQuestion->Items->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionResultatRetourAutoExecute(TObject *Sender)
{
 ActionResultatRetourAuto->Checked^=true;
 MemoReponse->ScrollBars=ActionResultatRetourAuto->Checked?ssVertical:ssBoth;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionResultatPoliceExecute(TObject *Sender)
{
 FontDialog->Font=MemoReponse->Font;
 if (FontDialog->Execute())
  MemoReponse->Font=FontDialog->Font;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionToutDevelopperExecute(TObject *Sender)
{
 if (TreeViewBase->Focused()) TreeViewBase->Selected->Expand(ALL_THE_NODES);
 if (TreeViewRecherche->Focused()) TreeViewRecherche->Selected->Expand(ALL_THE_NODES);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionToutReduireExecute(TObject *Sender)
{
 if (TreeViewBase->Focused()) TreeViewBase->Selected->Collapse(ALL_THE_NODES);
 if (TreeViewRecherche->Focused()) TreeViewRecherche->Selected->Collapse(ALL_THE_NODES);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxQuestionKeyPress(TObject *Sender,
      char &Key)
{
 if (Key=='\r')
  ActionRepondre->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionNumeroterExecute(TObject *Sender)
{
 ActionNumeroter->Checked^=true;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ActionEnregistrerVueArborescenteSousExecute(
      TObject *Sender)
{ // v0.3.4
 if (TreeViewBase->Focused())
  {
   SaveDialogVueArborescente->Title="Enregistrer la vue arborescente de la BASE sous...";
   SaveDialogVueArborescente->FileName="VueArborescenteBase.txt";
   if (SaveDialogVueArborescente->Execute())
    TreeViewBase->SaveToFile(AnsiString(SaveDialogVueArborescente->FileName));
  }
 if (TreeViewRecherche->Focused())
  {
   SaveDialogVueArborescente->Title="Enregistrer la vue arborescente de la RECHERCHE sous...";
   SaveDialogVueArborescente->FileName="VueArborescenteRecherche.txt";
   if (SaveDialogVueArborescente->Execute())
    TreeViewRecherche->SaveToFile(AnsiString(SaveDialogVueArborescente->FileName));
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
#ifdef PRFD_WINDOWS
 delete IniFile;
#endif
}
//---------------------------------------------------------------------------
#ifdef PRFD_WINDOWS
void TFormMain::LitHistoriqueQuestions(const AnsiString asNomFichier) // v0.3.2
 {
  if (!asNomFichier.IsEmpty())
   {
    ComboBoxQuestion->Items->Clear();
    bool Drn=false;
    for(int i=0; !Drn; i++)

     {
      AnsiString asQuestion= IniFile->ReadString( asNomFichier,
                                                  Format( "Question%3.3d",
                                                          ARRAYOFCONST((i+1))),
                                                  asVide);
      if (!(Drn=(asQuestion==asVide)))
       ComboBoxQuestion->Items->Add(asQuestion);
     }
   }
 }
//---------------------------------------------------------------------------
void TFormMain::EnregistreHistoriqueQuestions() // v0.3.2
 {
  if (!asNomFichier.IsEmpty())
   for(int i=0; i<ComboBoxQuestion->Items->Count; i++)
    IniFile->WriteString(asNomFichier, Format("Question%3.3d", ARRAYOFCONST((i+1))), ComboBoxQuestion->Items->Strings[i]);
 }
//---------------------------------------------------------------------------
#endif


